@@ -13,7 +13,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import collections
+import collections.abc
 import logging
 
 from openstack import exceptions as sdk_exc
@@ -28,21 +28,22 @@ LOG = logging.getLogger(__name__)
 class NICs(object):
     """Requested NICs."""
 
-    def __init__(self, connection, node, nics):
+    def __init__(self, connection, node, nics, hostname=None):
         if nics is None:
             nics = []
 
-        if not isinstance(nics, collections.Sequence):
+        if not isinstance(nics, collections.abc.Sequence):
             raise TypeError("NICs must be a list of dicts")
 
         for nic in nics:
-            if not isinstance(nic, collections.Mapping):
+            if not isinstance(nic, collections.abc.Mapping):
                 raise TypeError("Each NIC must be a dict got %s" % nic)
 
         self._node = node
         self._connection = connection
         self._nics = nics
         self._validated = None
+        self._hostname = hostname
         self.created_ports = []
         self.attached_ports = []
 
@@ -72,13 +73,22 @@ class NICs(object):
 
         for nic_type, nic in self._validated:
             if nic_type != 'port':
-                port = self._connection.network.create_port(**nic)
+                # The 'binding:host_id' must be set to ensure IP allocation
+                # is not deferred.
+                # See: https://storyboard.openstack.org/#!/story/2009715
+                port = self._connection.network.create_port(
+                    binding_host_id=self._node.id, **nic)
                 self.created_ports.append(port.id)
                 LOG.info('Created port %(port)s for node %(node)s with '
                          '%(nic)s', {'port': _utils.log_res(port),
                                      'node': _utils.log_res(self._node),
                                      'nic': nic})
             else:
+                # The 'binding:host_id' must be set to ensure IP allocation
+                # is not deferred.
+                # See: https://storyboard.openstack.org/#!/story/2009715
+                self._connection.network.update_port(
+                    nic, binding_host_id=self._node.id)
                 port = nic
 
             self._connection.baremetal.attach_vif_to_node(self._node,
@@ -137,6 +147,8 @@ class NICs(object):
         port_args = {'network_id': network.id}
         if nic.get('fixed_ip'):
             port_args['fixed_ips'] = [{'ip_address': nic['fixed_ip']}]
+        if self._hostname:
+            port_args['name'] = '%s-%s' % (self._hostname, network.name)
 
         return port_args
 
@@ -166,8 +178,11 @@ class NICs(object):
                 'Cannot find network %(net)s for subnet %(sub)s: %(error)s' %
                 {'net': subnet.network_id, 'sub': nic['subnet'], 'error': exc})
 
-        return {'network_id': network.id,
-                'fixed_ips': [{'subnet_id': subnet.id}]}
+        port_args = {'network_id': network.id,
+                     'fixed_ips': [{'subnet_id': subnet.id}]}
+        if self._hostname:
+            port_args['name'] = '%s-%s' % (self._hostname, network.name)
+        return port_args
 
 
 def detach_and_delete_ports(connection, node, created_ports, attached_ports):
